local AddonName, a = ...
local s = SpellFlashAddon
local c = BittensSpellFlashLibrary

local GetItemCount = GetItemCount
local GetSpellBookItemInfo = GetSpellBookItemInfo
local GetTime = GetTime
local IsMounted = IsMounted
local SPELL_POWER_SOUL_SHARDS = SPELL_POWER_SOUL_SHARDS
local UnitExists = UnitExists

c.Init(a)

------------------------------------------------------------------------ Common
a.spells["Power Torrent"] = { ID = 74241 }
a.spells["Soulburn"] = { ID = 74434 }

local soulFireOnly = { 6353 }
local soulFireAndSoulBurn = { 6353, 74434 }

a.EquipmentSets = {}
a.EquipmentSets.T13 = {
    HeadSlot = { 78797, 76342, 78702 },
    ShoulderSlot = { 78844, 76339, 78749 },
    ChestSlot = { 78825, 76340, 78730 },
    HandsSlot = { 78776, 76343, 78681 },
    LegsSlot = { 78816, 76341, 78721 },
}

local function getCurrentPet()
    for i = 1, 10000 do
        local _, spellID = GetSpellBookItemInfo(i, "pet")
        if spellID == nil then
            return nil
        elseif spellID == 54049 then -- Shadow Bite
            return "Felhunter"
        elseif spellID == 30213 then -- Legion Strike
            return "Felguard"
        elseif spellID == 3110 then -- Firebolt
            return "Imp"
        end
    end
end

a.spells["Bane of Doom"] = {
    ID = 603,
    MyDebuff = 603,
    Continue = 1,
    FlashColor = "yellow",
    CheckFirst = function()
       return not c.HasMyDebuff("Bane of Agony")
    end
}

a.spells["Refresh Bane of Doom"] = {
    ID = 603,
    MyDebuff = "Bane of Doom",
    Continue = 1,
    FlashColor = "yellow",
    EarlyRefresh = 14.9,
    CheckFirst = function()
       return not c.HasMyDebuff("Bane of Agony")
    end
}

a.spells["Corruption"] = {
    ID = 172,
    MyDebuff = 172,
}
c.ManageDotRefresh("Corruption", 3)

a.spells["Shadowflame"] = {
    ID = 47897,
    Melee = 1,
}

a.spells["Incinerate"] = {
    ID = 29722,
}

a.spells["Curse of the Elements"] = {
    ID = 1490,
    Debuff = c.SPELL_DAMAGE_DEBUFFS,
    Continue = 1,
    FlashColor = "yellow",
    EarlyRefresh = 3,
}

a.spells["Soulburn"] = {
    ID = 74434,
}

a.spells["Dark Intent"] = {
    ID = 80398,
    FlashColor = "yellow",
    Override = function()
        if not s.InRaidOrParty() then
            return not c.HasBuff("Dark Intent")
        end
        
        if a.DarkIntentTarget == nil then
            return true
        end
        
        if s.InCombat() then
            return not s.MyBuff(80398, a.DarkIntentTarget, c.GetBusyTime())
        else
            return not s.MyBuff(80398, a.DarkIntentTarget, 5 * 60)
        end
    end
}

a.spells["Summon Felhunter"] = {
    ID = 691,
    Continue = 1,
    FlashColor = "yellow",
    NotIfActive = 1,
    CheckFirst = function()
        return not IsMounted() and getCurrentPet() ~= "Felhunter"
    end
}

a.spells["Create Soulstone"] = {
    ID = 693,
    FlashColor = "yellow",
    NotWhileActive = true,
    CheckFirst = function()
        return GetItemCount(5232) == 0
    end
}

a.spells["Life Tap"] = {
    ID = 1454,
    CheckFirst = function()
       	return s.Power("player") < 12000
    end
}

a.spells["Life Tap out of Combat"] = {
    ID = 1454,
    FlashColor = "yellow",
    CheckFirst = function()
       	return s.PowerMissing("player") > 30000
    end
}

a.spells["Soul Harvest"] = {
    ID = 79268,
    FlashColor = "yellow",
    CheckFirst = function()
    	return s.Power("player", SPELL_POWER_SOUL_SHARDS) < 3
    		or s.HealthPercent("player") < 55
    end
}

a.spells["Demon Soul"] = {
    ID = 77801,
    Continue = 1,
    FlashColor = "yellow",
}

a.spells["Demon Soul Felguard"] = {
    ID = 77801,
    FlashColor = "yellow",
    CheckFirst = function()
        return getCurrentPet() == "Felguard"
    end
}

a.spells["Shadow Bolt"] = {
    ID = 686,
}
s.SetOtherAuras(c.GetID("Shadow Bolt"), a.TalentIDs["Shadow and Flame"])

a.spells["Fel Armor"] = {
    ID = 28176,
    Buff = 28176,
    BuffUnit = "player",
    FlashColor = "yellow",
}

a.spells["Soul Link"] = {
    ID = 19028,
    Buff = 19028,
    BuffUnit = "player",
    FlashColor = "yellow",
    CheckFirst = function()
       return UnitExists("pet")
    end
}

a.spells["Summon Doomguard"] = {
    ID = 18540,
    Continue = 1,
    FlashColor = "yellow",
    NotIfActive = 1,
    CheckFirst = function()
       return c.HasBuff("Power Torrent")
    end
}

local function burningFireCheck()
	return (s.Castable(a.spells["Soulburn"]) or c.HasBuff("Soulburn"))
		and c.GetCastTime("Soul Fire") > 1
end

a.spells["Soul Fire with T13"] = {
	ID = 6353,
	FlashID = soulFireAndSoulBurn,
	CheckFirst = function()
		return c.WearingSet(4, "T13") and burningFireCheck()
	end
}

a.spells["Soul Fire and Soulburn"] = {
	ID = 6353,
	FlashID = soulFireAndSoulBurn,
	CheckFirst = function()
		return burningFireCheck() and getCurrentPet() == "Felhunter"
	end
}

a.spells["Axe Toss"] = {
	Type = "pet",
	ID = 89766,
	Interrupt = true,
	FlashColor = "aqua",
}

a.spells["Spell Lock"] = {
	Type = "pet",
	ID = 19647,
	Interrupt = true,
	FlashColor = "aqua",
}

-------------------------------------------------------------------- Affliction
a.spells["Unstable Affliction"] = {
    ID = 30108,
    MyDebuff = 30108,
}
c.ManageDotRefresh("Unstable Affliction", 3)

a.spells["Haunt"] = {
    ID = 48181,
    NotIfActive = 1,
}

a.spells["Haunt for Debuff"] = {
    ID = 48181,
    MyDebuff = 48181,
    NotIfActive = 1,
}

a.spells["Drain Soul"] = {
    ID = 1120,
    CheckFirst = function()
        return s.HealthPercent() < 25
    end
}

a.spells["Shadow Bolt for Shadow and Flame"] = {
    ID = 686,
    Debuff = c.CRIT_DEBUFFS,
    EarlyRefresh = 5,
    CheckFirst = function()
       return c.HasTalent("Shadow and Flame")
    end
}

a.spells["Corruption under Haunt"] = {
    ID = 172,
    MyDebuff = 172,
    CheckFirst = function()
        return c.HasMyDebuff("Haunt")
    end
}
c.ManageDotRefresh("Corruption under Haunt", 3)

a.spells["Bane of Agony"] = {
    ID = 980,
    MyDebuff = 980,
    FlashColor = "yellow",
    Continue = 1,
    CheckFirst = function()
       return not c.HasMyDebuff("Bane of Doom")
    end
}
c.ManageDotRefresh("Bane of Agony", 2)

a.spells["Soul Swap"] = {
    ID = 86121,
    Continue = 1,
    FlashColor = "yellow",
    CheckFirst = function()
        return c.HasMyDebuff("Corruption")
            and c.HasMyDebuff("Unstable Affliction")
            and c.HasMyDebuff("Bane of Agony")
    end
}

-------------------------------------------------------------------- Demonology
a.spells["Molten Core"] = { ID = 71165 }

a.spells["Immolate"] = {
    ID = 348,
    MyDebuff = 348,
    CheckFirst = function()
        return GetTime() - a.ImmolateRefresh > 1
            and 0 == c.CountLandings(
	            "Hand of Gul'dan",
	            -3,
	            s.MyDebuffDuration(348))
    end
}
c.ManageDotRefresh("Immolate", 3)

a.spells["Hand of Gul'dan"] = {
    ID = 71521,
    NotIfActive = 1,
    TravelTimeEstimate = .25,
}

a.spells["Soul Fire under Decimation"] = {
    ID = 6353,
    RequireBuff = 63167, -- Decimation
    RequireBuffUnit = "player",
}

a.spells["Metamorphosis"] = {
    ID = 47241,
    FlashColor = "yellow",
    Continue = 1,
    NoGCD = 1,
}

a.spells["Immolation Aura"] = {
    ID = 50589,
    Melee = 1,
    Continue = 1,
    FlashColor = "yellow",
    CheckFirst = function()
        return c.GetBuffDuration("Metamorphosis") > 6
            and c.GetCooldown("Demon Soul") > 15
    end
}

a.spells["Summon Felguard"] = {
    ID = 30146,
    FlashColor = "yellow",
    NotIfActive = 1,
    CheckFirst = function()
        return not IsMounted() and getCurrentPet() ~= "Felguard"
    end
}

a.spells["Summon Felhunter in Combat"] = {
    ID = 691,
	FlashID = { 691, 74434 }, -- Soulburn
    Continue = 1,
    FlashColor = "yellow",
    NotIfActive = 1,
    CheckFirst = function()
        return getCurrentPet() ~= "Felhunter"
            and not (getCurrentPet() == "Felguard" 
                and (c.GetCooldown("Felstorm") == 0 
                    or s.Buff("Felstorm", "pet", c.GetBusyTime())))
    end
}

a.spells["Felstorm"] = {
    ID = 89751,
    NoGCD = 1,
    FlashColor = "yellow",
    CheckFirst = function()
        return c.GetCooldown("Demon Soul") > 15
    end
}

a.spells["Incinerate under Molten Core"] = {
    ID = 29722,
    CheckFirst = function()
        local stack = c.GetBuffStack("Molten Core")
        if c.IsCasting("Incinerate") then
            stack = stack - 1
        end
        return stack > 0
    end
}

a.spells["Demon Soul Felhunter"] = {
    ID = 77801,
    FlashColor = "yellow",
    CheckFirst = function()
        return getCurrentPet() == "Felhunter"
    end
}

------------------------------------------------------------------- Destruction
a.spells["Empowered Imp"] = { ID = 47283 }
a.spells["Improved Soul Fire"] = { ID = 85383 }

a.spells["Soul Fire"] = {
    ID = 6353,
    TravelTimeEstimate = 1,
}
c.RegisterAura("Soul Fire", "Improved Soul Fire")

a.spells["Soul Fire for Improved Soul Fire"] = {
    ID = 6353,
    Buff = c.GetID("Improved Soul Fire"),
    BuffUnit = "player",
    EarlyRefresh = 2,
    CheckFirst = function(z)
        if not c.HasTalent("Improved Soul Fire")
            or c.IsAuraPendingFor("Soul Fire") then
            
            return false
        end
        
        z.CastTime = c.EstimateTravelTime("Soul Fire")
        if c.HasBuff("Empowered Imp") then
            z.FlashID = soulFireOnly
        elseif c.GetCooldown("Soulburn") == 0
            and s.UsableSpell(c.GetID("Soulburn")) then
            
            z.FlashID = soulFireAndSoulBurn
        else
            z.FlashID = soulFireOnly
            z.CastTime = z.CastTime + c.GetCastTime("Soul Fire")
        end
        return true
    end
}

a.spells["Soul Fire under Empowered Imp"] = {
    ID = 6353,
    CheckFirst = function()
        return c.HasBuff("Empowered Imp")
    end
}

a.spells["Conflagrate"] = {
    ID = 17962,
    CheckFirst = function()
        return c.HasMyDebuff("Immolate")
    end
}

a.spells["Shadowburn"] = {
    ID = 17877,
}

a.spells["Chaos Bolt"] = {
    ID = 50796,
    NotIfActive = 1,
    CheckFirst = function()
        return c.GetCastTime("Chaos Bolt") >= 1
    end
}

a.spells["Summon Imp"] = {
    ID = 688,
    NotIfActive = 1,
    CheckFirst = function()
        return not IsMounted() and getCurrentPet() ~= "Imp"
    end
}
